@@ -195,6 +195,7 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_ADD$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_AUTOPANNER$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_MUL$");
+	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_DIV$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_MULTI_ADD$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_XFADE$");
 	menumaker->addCategory(i18n("&Synthesis/&Busses"), "Arts::Synth_BUS_");
@@ -753,7 +754,7 @@ void ArtsBuilderWindow::addInterface()
 	ifd->exec();
 
 	Arts::ModuleInfo minfo = makeModuleInfo(ifd->interfaceName());
-	if(minfo.name != "")
+	if(!minfo.name.empty())
 		modulewidget->addInterface(minfo);
 
 	delete ifd;
@@ -786,7 +787,7 @@ void ArtsBuilderWindow::addModule(const char *name)
 	arts_debug("addModule(%s)", name);
 	Arts::ModuleInfo minfo = makeModuleInfo(name);
 
-	if(minfo.name != "")
+	if(!minfo.name.empty())
 		modulewidget->addModule(minfo);
 #if 0
 	const char *name = menumaker->findID(module);
@@ -837,7 +838,7 @@ bool ArtsBuilderWindow::promptToSave()
 		return true;
 
 	query = KMessageBox::warningYesNoCancel(this,
-		i18n("The current structure has been modified.\nWould you like to save it?"));
+		i18n("The current structure has been modified.\nWould you like to save it?"), QString::null, KStdGuiItem::save(), KStdGuiItem::discard());
 
 	result = false;
 	switch(query)
