@@ -49,6 +49,7 @@ private:
     void setupSystemTray();
     void setupGlobalAccels();
     void processArgs();
+    void createDirs();
 
     void keyPressEvent(QKeyEvent *);
 
@@ -65,11 +66,14 @@ private:
 
 private slots:
     void slotShowHide();
+    void slotAboutToQuit();
     void slotQuit();
     void slotToggleSystemTray(bool enabled);
     void slotEditKeys();
     void slotConfigureTagGuesser();
     void slotConfigureFileRenamer();
+    void slotUndo();
+    void slotCheckAlbumNextAction(bool albumRandomEnabled);
 
 private:
     PlaylistSplitter *m_splitter;
