@@ -28,6 +28,7 @@
 #include <kurl.h>
 #include <kio/netaccess.h>
 #include <kio/job.h>
+#include <dcopclient.h>
 
 #include <qheader.h>
 #include <qcursor.h>
@@ -60,6 +61,7 @@
 #include "deletedialog.h"
 #include "googlefetcher.h"
 #include "coverinfo.h"
+#include "coverdialog.h"
 #include "tagtransactionmanager.h"
 #include "cache.h"
 
@@ -127,6 +129,7 @@ private:
 ////////////////////////////////////////////////////////////////////////////////
 
 bool Playlist::m_visibleChanged = false;
+bool Playlist::m_shuttingDown = false;
 
 /**
  * Shared settings between the playlists.
@@ -170,9 +173,8 @@ Playlist::SharedSettings *Playlist::SharedSettings::m_instance = 0;
 
 Playlist::SharedSettings *Playlist::SharedSettings::instance()
 {
-    if(!m_instance)
-	m_instance = new SharedSettings;
-    return m_instance;
+    static SharedSettings settings;
+    return &settings;
 }
 
 void Playlist::SharedSettings::setColumnOrder(const Playlist *l)
@@ -413,6 +415,9 @@ Playlist::~Playlist()
 
     if(isVisible() && this != CollectionList::instance())
 	m_collection->raise(CollectionList::instance());
+
+    if(!m_shuttingDown)
+	m_collection->removePlaylist(this);
 }
 
 QString Playlist::name() const
@@ -568,8 +573,6 @@ void Playlist::clearItem(PlaylistItem *item, bool emitChanged)
     m_members.remove(item->file().absFilePath());
     m_search.clearItem(item);
 
-    if(!m_randomList.isEmpty() && !m_visibleChanged)
-        m_randomList.remove(item);
     m_history.remove(item);
 
     delete item;
@@ -662,12 +665,10 @@ void Playlist::setSearch(const PlaylistSearch &s)
     if(!m_searchEnabled)
 	return;
 
-    // Make sure that we only play visible items.
-
-    m_randomList.clear();
-
     setItemsVisible(s.matchedItems(), true);
     setItemsVisible(s.unmatchedItems(), false);
+
+    TrackSequenceManager::instance()->iterator()->playlistChanged();
 }
 
 void Playlist::setSearchEnabled(bool enabled)
@@ -799,6 +800,26 @@ void Playlist::slotRemoveCover()
 	refreshAlbums(items);
 }
 
+void Playlist::slotShowCoverManager()
+{
+    static CoverDialog *managerDialog = 0;
+
+    if(!managerDialog)
+	managerDialog = new CoverDialog(this);
+
+    managerDialog->show();
+}
+
+unsigned int Playlist::eligibleCoverItems(const PlaylistItemList &items)
+{
+    // This used to count the number of tracks with an artist and album, that
+    // is not strictly required anymore.  This may prove useful in the future
+    // so I'm leaving it in for now, right now we just mark every item as
+    // eligible.
+
+    return items.count();
+}
+
 void Playlist::slotAddCover(bool retrieveLocal)
 {
     PlaylistItemList items = selectedItems();
@@ -806,29 +827,45 @@ void Playlist::slotAddCover(bool retrieveLocal)
     if(items.isEmpty())
         return;
 
-    QImage image;
+    if(eligibleCoverItems(items) == 0) {
+	// No items in the list can be assigned a cover, inform the user and
+	// bail.
+
+	// KDE 4.0 Fix this string.
+	KMessageBox::sorry(this, i18n("None of the items you have selected can "
+		    "be assigned a cover.  A track must have both the Artist "
+		    "and Album tags set to be assigned a cover."));
+
+	return;
+    }
+
+    QPixmap newCover;
 
     if(retrieveLocal) {
         KURL file = KFileDialog::getImageOpenURL(
 	    ":homedir", this, i18n("Select Cover Image File"));
-        image = QImage(file.directory() + "/" + file.fileName());
+        newCover = QPixmap(file.directory() + "/" + file.fileName());
     }
     else {
         PlaylistItemList::Iterator it=items.begin();
-        image = GoogleFetcher((*it)->file()).pixmap().convertToImage();;
+        newCover = GoogleFetcher((*it)->file()).pixmap();
     }
 
-    if(image.isNull())
+    if(newCover.isNull())
         return;
 
-    refreshAlbums(items, image);
+    QString artist = items.front()->file().tag()->artist();
+    QString album = items.front()->file().tag()->album();
+
+    coverKey newId = CoverManager::addCover(newCover, artist, album);
+    refreshAlbums(items, newId);
 }
 
 void Playlist::slotGuessTagInfo(TagGuesser::Type type)
 {
     KApplication::setOverrideCursor(Qt::waitCursor);
     PlaylistItemList items = selectedItems();
-    setCanDeletePlaylist(false);
+    setDynamicListsFrozen(true);
 
     m_blockDataChanged = true;
 
@@ -846,7 +883,7 @@ void Playlist::slotGuessTagInfo(TagGuesser::Type type)
     m_blockDataChanged = false;
 
     dataChanged();
-    setCanDeletePlaylist(true);
+    setDynamicListsFrozen(false);
     KApplication::restoreOverrideCursor();
 }
 
@@ -883,9 +920,16 @@ void Playlist::slotShowPlaying()
     Playlist *l = playingItem()->playlist();
 
     l->clearSelection();
+
+    // Raise the playlist before selecting the items otherwise the tag editor
+    // will not update when it gets the selectionChanged() notification
+    // because it will think the user is choosing a different playlist but not
+    // selecting a different item.
+
+    m_collection->raise(l);
+
     l->setSelected(playingItem(), true);
     l->ensureItemVisible(playingItem());
-    m_collection->raise(l);
 }
 
 void Playlist::slotColumnResizeModeChanged()
@@ -934,14 +978,9 @@ void Playlist::removeFromDisk(const PlaylistItemList &items)
 
 		QString removePath = (*it)->file().absFilePath();
 		if((!shouldDelete && KIO::NetAccess::synchronousRun(KIO::trash(removePath), this)) ||
-		   (shouldDelete && QFile::remove(removePath))) {
-                    if(!m_randomList.isEmpty() && !m_visibleChanged)
-                        m_randomList.remove(*it);
+		   (shouldDelete && QFile::remove(removePath)))
+		{
 		    CollectionList::instance()->clearItem((*it)->collectionItem());
-
-		    // Get Orwellian and erase the song from history. :-)
-
-		    m_history.remove(*it);
 		}
 		else
 		    errorFiles.append((*it)->file().absFilePath());
@@ -977,9 +1016,43 @@ QDragObject *Playlist::dragObject(QWidget *parent)
     return drag;
 }
 
+void Playlist::contentsDragEnterEvent(QDragEnterEvent *e)
+{
+    KListView::contentsDragEnterEvent(e);
+
+    if(CoverDrag::canDecode(e)) {
+	setDropHighlighter(true);
+	setDropVisualizer(false);
+
+	e->accept();
+	return;
+    }
+
+    setDropHighlighter(false);
+    setDropVisualizer(true);
+
+    KURL::List urls;
+    if(!KURLDrag::decode(e, urls) || urls.isEmpty()) {
+	e->ignore();
+	return;
+    }
+
+    e->accept();
+    return;
+}
+
+bool Playlist::acceptDrag(QDropEvent *e) const
+{
+    return CoverDrag::canDecode(e) || KURLDrag::canDecode(e);
+}
+
 bool Playlist::canDecode(QMimeSource *s)
 {
     KURL::List urls;
+
+    if(CoverDrag::canDecode(s))
+	return true;
+
     return KURLDrag::decode(s, urls) && !urls.isEmpty();
 }
 
@@ -1015,7 +1088,7 @@ void Playlist::decode(QMimeSource *s, PlaylistItem *item)
     QStringList fileList;
 
     for(KURL::List::Iterator it = urls.begin(); it != urls.end(); ++it)
-	fileList.append((*it).path());
+	fileList += MediaFiles::convertURLsToLocal((*it).path(), this);
 
     addFiles(fileList, item);
 }
@@ -1047,7 +1120,6 @@ bool Playlist::eventFilter(QObject *watched, QEvent *e)
 		QTimer::singleShot(0, this, SLOT(slotUpdateColumnWidths()));
 	    break;
 	}
-
 	default:
 	    break;
 	}
@@ -1056,11 +1128,53 @@ bool Playlist::eventFilter(QObject *watched, QEvent *e)
     return KListView::eventFilter(watched, e);
 }
 
+void Playlist::keyPressEvent(QKeyEvent *event)
+{
+    if(event->key() == Key_Up) {
+	QListViewItemIterator selected(this, QListViewItemIterator::IteratorFlag(
+					   QListViewItemIterator::Selected |
+					   QListViewItemIterator::Visible));
+	if(selected.current()) {
+	    QListViewItemIterator visible(this, QListViewItemIterator::IteratorFlag(
+					      QListViewItemIterator::Visible));
+	    if(selected.current() == visible.current())
+		KApplication::postEvent(parent(), new FocusUpEvent);
+	}
+	
+    }
+
+    KListView::keyPressEvent(event);
+}
+
 void Playlist::contentsDropEvent(QDropEvent *e)
 {
     QPoint vp = contentsToViewport(e->pos());
     PlaylistItem *item = static_cast<PlaylistItem *>(itemAt(vp));
 
+    // First see if we're dropping a cover, if so we can get it out of the
+    // way early.
+    if(item && CoverDrag::canDecode(e)) {
+	coverKey id;
+	CoverDrag::decode(e, id);
+
+	// If the item we dropped on is selected, apply cover to all selected
+	// items, otherwise just apply to the dropped item.
+
+	if(item->isSelected()) {
+	    PlaylistItemList selItems = selectedItems();
+	    for(PlaylistItemList::Iterator it = selItems.begin(); it != selItems.end(); ++it) {
+		(*it)->file().coverInfo()->setCoverId(id);
+		(*it)->refresh();
+	    }
+	}
+	else {
+	    item->file().coverInfo()->setCoverId(id);
+	    item->refresh();
+	}
+	
+	return;
+    }
+
     // When dropping on the upper half of an item, insert before this item.
     // This is what the user expects, and also allows the insertion at
     // top of the list
@@ -1229,23 +1343,21 @@ void Playlist::addFiles(const QStringList &files, PlaylistItem *after)
     KApplication::restoreOverrideCursor();
 }
 
-void Playlist::refreshAlbums(const PlaylistItemList &items, const QImage &image)
+void Playlist::refreshAlbums(const PlaylistItemList &items, coverKey id)
 {
     QValueList< QPair<QString, QString> > albums;
+    bool setAlbumCovers = items.count() == 1;
 
     for(PlaylistItemList::ConstIterator it = items.begin(); it != items.end(); ++it) {
-	if(albums.find(qMakePair((*it)->file().tag()->artist(),
-				 (*it)->file().tag()->album())) == albums.end())
-	{
-	    albums.append(qMakePair((*it)->file().tag()->artist(),
-				    (*it)->file().tag()->album()));				    
-	    if(image.isNull())
-		(*it)->file().coverInfo()->clearCover();
-	    else
-		(*it)->file().coverInfo()->setCover(image);
-	}
-	else
-	    (*it)->file().coverInfo()->setCover();
+	QString artist = (*it)->file().tag()->artist();
+	QString album = (*it)->file().tag()->album();
+
+	if(albums.find(qMakePair(artist, album)) == albums.end())
+	    albums.append(qMakePair(artist, album));
+
+	(*it)->file().coverInfo()->setCoverId(id);
+	if(setAlbumCovers)
+	    (*it)->file().coverInfo()->applyCoverToWholeAlbum(true);
     }
 
     for(QValueList< QPair<QString, QString> >::ConstIterator it = albums.begin();
@@ -1333,7 +1445,11 @@ void Playlist::showColumn(int c, bool updateSearch)
     // Just set the width to one to mark the column as visible -- we'll update
     // the real size in the next call.
 
-    setColumnWidth(c, 1);
+    if(manualResize())
+	setColumnWidth(c, 35); // Make column at least slightly visible.
+    else
+	setColumnWidth(c, 1);
+
     header()->setResizeEnabled(true, c);
     header()->moveSection(c, c); // Approximate old position
 
@@ -1459,9 +1575,9 @@ void Playlist::setupItem(PlaylistItem *item)
     }
 }
 
-void Playlist::setCanDeletePlaylist(bool canDelete)
+void Playlist::setDynamicListsFrozen(bool frozen)
 {
-    m_collection->setCanDeletePlaylist(canDelete);
+    m_collection->setDynamicListsFrozen(frozen);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1571,6 +1687,8 @@ void Playlist::setPlaying(PlaylistItem *item, bool addToHistory)
     }
 
     TrackSequenceManager::instance()->setCurrent(item);
+    QByteArray data;
+    kapp->dcopClient()->emitDCOPSignal("Player", "trackChanged()", data);
 
     if(!item)
 	return;
@@ -1928,7 +2046,8 @@ void Playlist::slotShowRMBMenu(QListViewItem *item, const QPoint &point, int col
 	    action("edit_cut")->plug(m_rmbMenu);
 	    action("edit_copy")->plug(m_rmbMenu);
 	    action("edit_paste")->plug(m_rmbMenu);
-	    action("edit_clear")->plug(m_rmbMenu);
+	    m_rmbMenu->insertSeparator();
+	    action("removeFromPlaylist")->plug(m_rmbMenu);
 	}
 	else
 	    action("edit_copy")->plug(m_rmbMenu);
@@ -2084,7 +2203,7 @@ void Playlist::slotInlineEditDone(QListViewItem *, const QString &, int column)
 	   0,
 	   i18n("This will edit multiple files. Are you sure?"),
 	   QString::null,
-	   KStdGuiItem::cont(),
+	   i18n("Edit"),
 	   "DontWarnMultipleTags") == KMessageBox::Cancel))
     {
 	return;
