@@ -24,10 +24,12 @@
 #include <qvaluevector.h>
 #include <qfileinfo.h>
 
+#include "covermanager.h"
 #include "stringhash.h"
 #include "playlistsearch.h"
 #include "tagguesser.h"
 #include "playlistinterface.h"
+#include "playlistitem.h"
 
 class KPopupMenu;
 class KActionMenu;
@@ -36,11 +38,10 @@ class QEvent;
 
 class PlaylistCollection;
 
-class PlaylistItem;
 class PlaylistToolTip;
-typedef QValueList<PlaylistItem *> PlaylistItemList;
+class UpcomingPlaylist;
 
-typedef QValueList<Playlist *> PlaylistList;
+typedef QValueList<PlaylistItem *> PlaylistItemList;
 
 class Playlist : public KListView, public PlaylistInterface
 {
@@ -75,9 +76,17 @@ public:
     virtual void playPrevious();
     virtual void stop();
 
-    // Plays the top item of the playlist.
+    /**
+     * Plays the top item of the playlist.
+     */
     void playFirst();
 
+    /**
+     * Plays the next album in the playlist.  Only useful when in album random
+     * play mode.
+     */
+    void playNextAlbum();
+
     /**
      * Saves the file to the currently set file name.  If there is no filename
      * currently set, the default behavior is to prompt the user for a file
@@ -115,12 +124,12 @@ public:
      * @return 0 if no file is playing, otherwise a pointer to the PlaylistItem
      *     of the track that is currently playing.
      */
-    static PlaylistItem *playingItem() { return m_playingItem; }
+    static PlaylistItem *playingItem();
 
     /**
      * All of the (media) files in the list.
      */
-    QStringList files();
+    QStringList files() const;
 
     /**
      * Returns a list of all of the items in the playlist.
@@ -137,6 +146,11 @@ public:
      */
     PlaylistItemList selectedItems();
 
+    /**
+     * Returns properly casted first child item in list.
+     */
+    PlaylistItem *firstChild() const;
+
     /**
      * Allow duplicate files in the playlist.
      */
@@ -163,7 +177,7 @@ public:
 			 QListViewItem *after = 0,
 			 bool emitChanged = true);
 
-    virtual void createItems(const PlaylistItemList &siblings);
+    virtual void createItems(const PlaylistItemList &siblings, PlaylistItem *after = 0);
 
     /**
      * This handles adding files of various types -- music, playlist or directory
@@ -174,8 +188,7 @@ public:
      * items since it has the overhead of checking to see if the file is a playlist
      * or directory first.
      */
-    void addFiles(const QStringList &files, bool importPlaylists,
-		  PlaylistItem *after = 0);
+    virtual void addFiles(const QStringList &files, PlaylistItem *after = 0);
 
     /**
      * Returns the file name associated with this playlist (an m3u file) or
@@ -232,6 +245,11 @@ public:
      */
     void updateLeftColumn();
 
+    /**
+     * Returns the leftmost visible column of the listview.
+     */
+    int leftColumn() const { return m_leftColumn; }
+
     /**
      * Sets the items in the list to be either visible based on the value of
      * visible.  This is useful for search operations and such.
@@ -275,7 +293,23 @@ public:
      */
     virtual bool readOnly() const { return false; }
 
-    void setColumnWidthUpdatesDisabled(bool disabled) { m_disableColumnWidthUpdates = disabled; }
+    /**
+     * Returns true if it's possible to reload this playlist.
+     */
+    virtual bool canReload() const { return !m_fileName.isNull(); }
+
+    /**
+     * Returns true if the playlist is a search playlist and the search should be
+     * editable.
+     */
+    virtual bool searchIsEditable() const { return false; }
+
+    /**
+     * Synchronizes the the playing item in this playlist with the playing item
+     * in \a sources.  If \a setMaster is true, this list will become the source
+     * for determining the next item.
+     */
+    void synchronizePlayingItems(const PlaylistList &sources, bool setMaster);
 
     /**
      * Playlists have a common set of shared settings such as visible columns
@@ -286,17 +320,14 @@ public:
 
     void read(QDataStream &s);
 
+    static void setShuttingDown() { m_shuttingDown = true; }
+
 public slots:
     /**
      * Remove the currently selected items from the playlist and disk.
      */
     void slotRemoveSelectedItems() { removeFromDisk(selectedItems()); };
 
-    /**
-     * Set the first selected item to be the next item returned by nextItem().
-     */
-    void slotSetNext();
-
     /*
      * The edit slots are required to use the canonical names so that they are
      * detected by the application wide framework.
@@ -338,6 +369,27 @@ public slots:
      */
     void slotRenameFile();
 
+    /**
+     * Sets the cover of the selected items, pass in true if you want to load from the local system,
+     * false if you want to load from the internet.
+     */
+    void slotAddCover(bool fromLocal);
+
+    /**
+     * Shows a large image of the cover
+     */
+    void slotViewCover();
+
+    /**
+     * Removes covers from the selected items
+     */
+    void slotRemoveCover();
+
+    /**
+     * Shows the cover manager GUI dialog
+     */
+    void slotShowCoverManager();
+
     /**
      * Reload the playlist contents from the m3u file.
      */
@@ -352,6 +404,10 @@ public slots:
 
     void slotShowPlaying();
 
+    void slotColumnResizeModeChanged();
+
+    virtual void dataChanged();
+
 protected:
     /**
      * Remove \a items from the playlist and disk.  This will ignore items that
@@ -362,16 +418,24 @@ protected:
     // the following are all reimplemented from base classes
 
     virtual bool eventFilter(QObject *watched, QEvent *e);
+    virtual void keyPressEvent(QKeyEvent *e);
     virtual QDragObject *dragObject(QWidget *parent);
     virtual QDragObject *dragObject() { return dragObject(this); }
     virtual bool canDecode(QMimeSource *s);
-    virtual void decode(QMimeSource *s, PlaylistItem *after = 0);
+    virtual void decode(QMimeSource *s, PlaylistItem *item = 0);
     virtual void contentsDropEvent(QDropEvent *e);
+    virtual void contentsMouseDoubleClickEvent(QMouseEvent *e);
+    virtual void contentsDragEnterEvent(QDragEnterEvent *e);
     virtual void showEvent(QShowEvent *e);
-    virtual bool acceptDrag(QDropEvent *e) const { return KURLDrag::canDecode(e); }
+    virtual bool acceptDrag(QDropEvent *e) const;
     virtual void viewportPaintEvent(QPaintEvent *pe);
     virtual void viewportResizeEvent(QResizeEvent *re);
 
+    virtual void insertItem(QListViewItem *item);
+    virtual void takeItem(QListViewItem *item);
+
+    virtual bool hasItem(const QString &file) const { return m_members.contains(file); }
+
     void addColumn(const QString &label);
 
     /**
@@ -395,7 +459,10 @@ protected:
      * Forwards the call to the parent to enable or disable automatic deletion
      * of tree view playlists.  Used by CollectionListItem.
      */
-    void setCanDeletePlaylist(bool canDelete);
+    void setDynamicListsFrozen(bool frozen);
+
+    template <class ItemType, class SiblingType>
+    ItemType *createItem(SiblingType *sibling, ItemType *after = 0);
 
     /**
      * As a template this allows us to use the same code to initialize the items
@@ -403,7 +470,7 @@ protected:
      * ItemType should be a PlaylistItem subclass.
      */
     template <class CollectionItemType, class ItemType, class SiblingType>
-    void createItems(const QValueList<SiblingType *> &siblings);
+    void createItems(const QValueList<SiblingType *> &siblings, ItemType *after = 0);
 
 protected slots:
     void slotPopulateBackMenu() const;
@@ -427,16 +494,13 @@ signals:
 
     void signalEnableDirWatch(bool enable);
 
+    void coverChanged();
+
+    void signalPlaylistItemsDropped(Playlist *p);
+
 private:
     void setup();
 
-    PlaylistItem *nextItem(PlaylistItem *current = 0) const;
-
-    /** Sets up album random play to play songs with the same album as
-     * the given PlaylistItem.
-     */
-    void initAlbumSearch(const PlaylistItem *item);
-
     /**
      * Load the playlist from a file.  \a fileName should be the absolute path.
      * \a fileInfo should point to the same file as \a fileName.  This is a
@@ -473,10 +537,29 @@ private:
      */
     void calculateColumnWeights();
 
-    PlaylistItem *addFile(const QString &file, bool importPlaylists, PlaylistItem *after);
+    void addFile(const QString &file, FileHandleList &files, bool importPlaylists,
+		 PlaylistItem **after);
+    void addFileHelper(FileHandleList &files, PlaylistItem **after,
+		       bool ignoreTimer = false);
 
     void redisplaySearch() { setSearch(m_search); }
 
+    /**
+     * Sets the cover for items to the cover identified by id.
+     */
+    void refreshAlbums(const PlaylistItemList &items, coverKey id = CoverManager::NoMatch);
+
+    void refreshAlbum(const QString &artist, const QString &album);
+
+    /**
+     * Returns the number of PlaylistItems in @p items that can be assigned a
+     * cover.  Used to avoid wasting the users' time setting the cover for 20
+     * items when none are eligible.
+     */
+    unsigned int eligibleCoverItems(const PlaylistItemList &items);
+
+    void updatePlaying() const;
+
     /**
      * This class is used internally to store settings that are shared by all
      * of the playlists, such as column order.  It is implemented as a singleton.
@@ -487,6 +570,8 @@ private slots:
 
     void slotUpdateColumnWidths();
 
+    void slotAddToUpcoming();
+
     /**
      * Show the RMB menu.  Matches the signature for the signal 
      * QListView::contextMenuRequested().
@@ -550,14 +635,21 @@ private:
     int m_currentColumn;
     int m_processed;
     int m_rmbEditID;
+    int m_rmbUpcomingID;
     int m_selectedCount;
 
     bool m_allowDuplicates;
     bool m_polished;
     bool m_applySharedSettings;
+    bool m_mousePressed;
 
     QValueList<int> m_weightDirty;
     bool m_disableColumnWidthUpdates;
+
+    mutable int m_time;
+    mutable QValueList<PlaylistItem::Pointer> m_addTime;
+    mutable QValueList<PlaylistItem::Pointer> m_subtractTime;
+
     /**
      * The average minimum widths of columns to be used in balancing calculations.
      */
@@ -565,10 +657,8 @@ private:
     QValueVector<int> m_columnFixedWidths;
     bool m_widthsDirty;
 
-    PlaylistItemList m_randomList;
-    PlaylistItemList m_history;
+    static PlaylistItemList m_history;
     PlaylistSearch m_search;
-    PlaylistSearch m_albumSearch;
 
     bool m_searchEnabled;
 
@@ -598,15 +688,23 @@ private:
      * call to setVisibleItems()) while random play is playing.
      */
     static bool m_visibleChanged;
+    static bool m_shuttingDown;
     static int m_leftColumn;
-    static PlaylistItem *m_playingItem;
-    static PlaylistItem *m_playNextItem;
     static QMap<int, PlaylistItem *> m_backMenuItems;
+
+    bool m_blockDataChanged;
 };
 
 bool processEvents();
 
-QDataStream &operator<<(QDataStream &s, Playlist &p);
+class FocusUpEvent : public QCustomEvent
+{
+public:
+    FocusUpEvent() : QCustomEvent(id) {}
+    static const int id = 999;
+};
+
+QDataStream &operator<<(QDataStream &s, const Playlist &p);
 QDataStream &operator>>(QDataStream &s, Playlist &p);
 
 // template method implementations
@@ -636,9 +734,6 @@ ItemType *Playlist::createItem(const FileHandle &file, QListViewItem *after,
 	ItemType *i = after ? new ItemType(item, this, after) : new ItemType(item, this);
 	setupItem(i);
 
-        if(!m_randomList.isEmpty() && !m_visibleChanged)
-            m_randomList.append(i);
-
 	if(emitChanged)
 	    dataChanged();
 
@@ -648,27 +743,32 @@ ItemType *Playlist::createItem(const FileHandle &file, QListViewItem *after,
 	return 0;
 }
 
+template <class ItemType, class SiblingType>
+ItemType *Playlist::createItem(SiblingType *sibling, ItemType *after)
+{
+    m_disableColumnWidthUpdates = true;
+    
+    if(!m_members.insert(sibling->file().absFilePath()) || m_allowDuplicates) {
+	after = new ItemType(sibling->collectionItem(), this, after);
+	setupItem(after);
+    }
+
+    m_disableColumnWidthUpdates = false;
+
+    return after;
+}
+
 template <class CollectionItemType, class ItemType, class SiblingType>
-void Playlist::createItems(const QValueList<SiblingType *> &siblings)
+void Playlist::createItems(const QValueList<SiblingType *> &siblings, ItemType *after)
 {
     if(siblings.isEmpty())
 	return;
 
-    m_disableColumnWidthUpdates = true;
-    ItemType *newItem = 0;
-
     QValueListConstIterator<SiblingType *> it = siblings.begin();
-    for(; it != siblings.end(); ++it) {
-	if(!m_members.insert((*it)->file().absFilePath()) || m_allowDuplicates) {
-	    newItem = new ItemType((*it)->collectionItem(), this, newItem);
-	    setupItem(newItem);
-	    if(!m_randomList.isEmpty() && !m_visibleChanged)
-		m_randomList.append(newItem);
-	}
-    }
+    for(; it != siblings.end(); ++it)
+	after = createItem(*it, after);
 
     dataChanged();
-    m_disableColumnWidthUpdates = false;
     slotWeightDirty();
 }
 
