@@ -24,6 +24,7 @@
 #include "playlistcollection.h"
 #include "actioncollection.h"
 #include "advancedsearchdialog.h"
+#include "coverinfo.h"
 #include "searchplaylist.h"
 #include "folderplaylist.h"
 #include "historyplaylist.h"
@@ -79,6 +80,7 @@ PlaylistCollection::~PlaylistCollection()
     saveConfig();
     delete m_actionHandler;
     PlayerManager::instance()->setPlaylistInterface(0);
+    Playlist::setShuttingDown();
 }
 
 QString PlaylistCollection::name() const
@@ -209,10 +211,14 @@ void PlaylistCollection::showMore(const QString &artist, const QString &album)
     else
         m_showMorePlaylist = new SearchPlaylist(this, search, i18n("Now Playing"), false, true);
 
-    m_belowShowMorePlaylist = visiblePlaylist();
+    // The call to raise() below will end up clearing m_belowShowMorePlaylist,
+    // so cache the value we want it to have now.
+    Playlist *belowShowMore = visiblePlaylist();
 
     PlaylistCollection::setupPlaylist(m_showMorePlaylist, QString::null);
     PlaylistCollection::raise(m_showMorePlaylist);
+
+    m_belowShowMorePlaylist = belowShowMore;
 }
 
 void PlaylistCollection::removeTrack(const QString &playlist, const QStringList &files)
@@ -270,6 +276,23 @@ QString PlaylistCollection::trackProperty(const QString &file, const QString &pr
     return item ? item->file().property(property) : QString::null;
 }
 
+QPixmap PlaylistCollection::trackCover(const QString &file, const QString &size) const
+{
+    if(size.lower() != "small" && size.lower() != "large")
+        return QPixmap();
+
+    CollectionList *l = CollectionList::instance();
+    CollectionListItem *item = l->lookup(file);
+
+    if(!item)
+        return QPixmap();
+
+    if(size.lower() == "small")
+        return item->file().coverInfo()->pixmap(CoverInfo::Thumbnail);
+    else
+        return item->file().coverInfo()->pixmap(CoverInfo::FullSize);
+}
+
 void PlaylistCollection::open(const QStringList &l)
 {
     QStringList files = l;
@@ -435,6 +458,11 @@ void PlaylistCollection::viewCovers()
     visiblePlaylist()->slotViewCover();
 }
 
+void PlaylistCollection::showCoverManager()
+{
+    visiblePlaylist()->slotShowCoverManager();
+}
+
 PlaylistItemList PlaylistCollection::selectedItems()
 {
     return visiblePlaylist()->selectedItems();
@@ -856,6 +884,8 @@ PlaylistCollection::ActionHandler::ActionHandler(PlaylistCollection *collection)
         SLOT(slotAddInternetCover()), "googleCover", "connect_established", "CTRL+SHIFT+g"));
     menu->insert(createAction(i18n("&Delete Cover"),
         SLOT(slotRemoveCovers()), "removeCover", "editdelete"));
+    menu->insert(createAction(i18n("Show Cover &Manager"),
+        SLOT(slotShowCoverManager()), "showCoverManager"));
 
     KToggleAction *historyAction =
         new KToggleAction(i18n("Show &History"), "history",  0, actions(), "showHistory");
