@@ -26,6 +26,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <iostream>
+#include <assert.h>
+#include <qsocketnotifier.h>
 
 extern "C"
 {
@@ -38,30 +40,31 @@ extern "C"
 
 // Local Headers
 #include "mixer_alsa.h"
+//#include "mixer.h"
 #include "volume.h"
 // #define if you want MUCH debugging output
 //#define ALSA_SWITCH_DEBUG
 //#define KMIX_ALSA_VOLUME_DEBUG
 
-// @todo Add file descriptor based notifying for seeing changes
-
-Mixer*
-ALSA_getMixer( int device, int card )
+Mixer_Backend*
+ALSA_getMixer( int device )
 {
-	Mixer *l_mixer;
-	l_mixer = new Mixer_ALSA( device, card );
+	Mixer_Backend *l_mixer;
+        l_mixer = new Mixer_ALSA( device );
 	return l_mixer;
 }
 
-Mixer_ALSA::Mixer_ALSA( int device, int card ) :
-	Mixer( device, card ), _handle(0)
+Mixer_ALSA::Mixer_ALSA( int device ) : Mixer_Backend( device )
 {
-	masterChosen = false;
-        _initialUpdate = true;
+    m_fds = 0;
+    m_sns = 0;
+    _handle = 0;
+    _initialUpdate = true;
 }
 
 Mixer_ALSA::~Mixer_ALSA()
 {
+  close();
 }
 
 int
@@ -69,14 +72,7 @@ Mixer_ALSA::identify( snd_mixer_selem_id_t *sid )
 {
 	QString name = snd_mixer_selem_id_get_name( sid );
 
-	if ( name == "Master" )
-	{
-		if (!masterChosen) {
-			m_masterDevice = snd_mixer_selem_id_get_index( sid );
-			masterChosen = true; // -<- this makes KMix select the *first* master device
-		}
-		return MixDevice::VOLUME;
-	}
+	if ( name == "Master" ) return MixDevice::VOLUME;
         if ( name == "Capture" ) return MixDevice::RECMONITOR;
 	if ( name == "Master Mono" ) return MixDevice::VOLUME;
         if ( name == "PC Speaker" ) return MixDevice::VOLUME;
@@ -102,14 +98,13 @@ Mixer_ALSA::identify( snd_mixer_selem_id_t *sid )
 }
 
 int
-Mixer_ALSA::openMixer()
+Mixer_ALSA::open()
 {
     bool virginOpen = m_mixDevices.isEmpty();
     bool validDevice = false;
+    bool masterChosen = false;
     int err;
 
-    release();
-
     snd_ctl_t *ctl_handle;
     snd_ctl_card_info_t *hw_info;
     snd_ctl_card_info_alloca(&hw_info);
@@ -119,7 +114,6 @@ Mixer_ALSA::openMixer()
     snd_mixer_selem_id_alloca( &sid );
 
     // Card information
-    QString devName;
     if( m_devnum == -1 )
         m_devnum = 0;
     if ( (unsigned)m_devnum > 31 )
@@ -128,7 +122,7 @@ Mixer_ALSA::openMixer()
 	devName = QString( "hw:%1" ).arg( m_devnum );
 
     QString probeMessage;
-	
+
     if (virginOpen)
 	probeMessage += "Trying ALSA Device '" + devName + "': ";
 
@@ -156,12 +150,16 @@ Mixer_ALSA::openMixer()
     snd_ctl_close( ctl_handle );
 
     /* open mixer device */
+
+    //kdDebug(67100) << "IN  Mixer_ALSA snd_mixer_open()" << endl;
     if ( ( err = snd_mixer_open ( &_handle, 0 ) ) < 0 )
     {
 	kdDebug(67100) << probeMessage << "not found: snd_mixer_open err=" << snd_strerror(err) << endl;
 	//errormsg( Mixer::ERR_NODEV );
+	_handle = 0;
 	return Mixer::ERR_NODEV; // if we cannot open the mixer, we have no devices
     }
+    //kdDebug(67100) << "OUT Mixer_ALSA snd_mixer_open()" << endl;
 
     if ( ( err = snd_mixer_attach ( _handle, devName.latin1() ) ) < 0 )
     {
@@ -181,17 +179,21 @@ Mixer_ALSA::openMixer()
     {
 	kdDebug(67100) << probeMessage << "not found: snd_mixer_load err=" << snd_strerror(err) << endl;
 	//errormsg( Mixer::ERR_READ );
-	releaseMixer();
+	close();
 	return Mixer::ERR_READ;
     }
 
-    kdDebug(67100) << probeMessage << "found" << endl;	
+    kdDebug(67100) << probeMessage << "found" << endl;
 
     unsigned int mixerIdx = 0;
     for ( elem = snd_mixer_first_elem( _handle ); elem; elem = snd_mixer_elem_next( elem ), mixerIdx++ )
     {
 	// If element is not active, just skip
 	if ( ! snd_mixer_selem_is_active ( elem ) ) {
+	    // ...but we still want to insert a null value into our mixer element
+	    // list so that the list indexes match up.
+	    mixer_elem_list.append( 0 );
+	    mixer_sid_list.append( 0 );
 	    continue;
 	}
 
@@ -209,24 +211,59 @@ Mixer_ALSA::openMixer()
 	snd_mixer_selem_get_capture_volume_range( elem, &minVolumeRec , &maxVolumeRec  );
 	// New mix device
 	MixDevice::ChannelType ct = (MixDevice::ChannelType)identify( sid );
-
+/*
+        if (!masterChosen && ct==MixDevice::VOLUME) {
+           // Determine a nicer MasterVolume
+	   m_masterDevice = mixerIdx;
+           masterChosen = true;
+        }
+*/
 	if( virginOpen )
 	{
-		MixDevice::DeviceCategory cc = MixDevice::UNDEFINED;
-		
-		kdDebug() << "--- Loop: name=" << snd_mixer_selem_id_get_name( sid ) << " , mixerIdx=" << mixerIdx << "------------" << endl;
-		
+	    MixDevice::DeviceCategory cc = MixDevice::UNDEFINED;
+
+		//kdDebug() << "--- Loop: name=" << snd_mixer_selem_id_get_name( sid ) << " , mixerIdx=" << mixerIdx << "------------" << endl;
+
+	    Volume* vol = 0;
+	    QPtrList<QString> enumList;
+	    if ( snd_mixer_selem_is_enumerated(elem) ) {
+		cc = MixDevice::ENUM;
+		vol = new Volume(); // Dummy, unused
+		mixer_elem_list.append( elem );
+		mixer_sid_list.append( sid );
+
+		// --- get Enum names START ---
+		int numEnumitems = snd_mixer_selem_get_enum_items(elem);
+		if ( numEnumitems > 0 ) {
+		  // OK. no error
+		  for (int iEnum = 0; iEnum<numEnumitems; iEnum++ ) {
+		    char buffer[100];
+		    int ret = snd_mixer_selem_get_enum_item_name(elem, iEnum, 99, buffer);
+		    if ( ret == 0 ) {
+		      QString* enumName = new QString(buffer);
+		      //enumName->append(buffer);
+		      enumList.append( enumName);
+		    } // enumName could be read succesfully
+		  } // for all enum items of this device
+		} // no error in reading enum list
+		else {
+		  // 0 items or Error code => ignore this entry
+		}
+		// --- get Enum names END ---
+	    } // is an enum
+
+	    else {
 		Volume::ChannelMask chn = Volume::MNONE;
 		Volume::ChannelMask chnTmp;
 		if ( snd_mixer_selem_has_playback_volume(elem) ) {
-			kdDebug(67100) << "has_playback_volume()" << endl;
+			//kdDebug(67100) << "has_playback_volume()" << endl;
 			chnTmp = snd_mixer_selem_is_playback_mono ( elem )
 				? Volume::MLEFT : (Volume::ChannelMask)(Volume::MLEFT | Volume::MRIGHT);
 			chn = (Volume::ChannelMask) (chn | chnTmp);
 			cc = MixDevice::SLIDER;
 		}
 		else if ( snd_mixer_selem_has_capture_volume(elem) ) {
-			kdDebug(67100) << "has_capture_volume()" << endl;
+			//kdDebug(67100) << "has_capture_volume()" << endl;
 			chnTmp = snd_mixer_selem_is_capture_mono( elem )
 				? Volume::MLEFT : (Volume::ChannelMask)(Volume::MLEFT | Volume::MRIGHT );
 			chn = (Volume::ChannelMask) (chn | chnTmp);
@@ -235,23 +272,23 @@ Mixer_ALSA::openMixer()
 			// It's not best coding ever, anyway
 			snd_mixer_selem_get_capture_volume_range( elem, &minVolumePlay, &maxVolumePlay );
 		}
-		
+
 		/* Create Volume object. If there is no volume on this device,
 		 * it will be created with maxVolume == 0 && minVolume == 0 */
-		Volume* vol = new Volume( chn, maxVolumePlay, minVolumePlay, maxVolumeRec, minVolumeRec );
-		//mixer_elem_list.append( elem );
+		vol = new Volume( chn, maxVolumePlay, minVolumePlay, maxVolumeRec, minVolumeRec );
+		mixer_elem_list.append( elem );
 		mixer_sid_list.append( sid );
-		
-		if ( snd_mixer_selem_has_playback_switch ( elem ) ) {   
-			kdDebug(67100) << "has_playback_switch()" << endl;
+
+		if ( snd_mixer_selem_has_playback_switch ( elem ) ) {
+			//kdDebug(67100) << "has_playback_switch()" << endl;
 			canMute = true;
 		}
 		if ( snd_mixer_selem_has_capture_switch ( elem ) )	{
-			kdDebug(67100) << "has_capture_switch()" << endl;
+			//kdDebug(67100) << "has_capture_switch()" << endl;
 			canRecord = true;
 		}
 		if ( snd_mixer_selem_has_common_switch ( elem ) ) {
-			kdDebug(67100) << "has_common_switch()" << endl;
+			//kdDebug(67100) << "has_common_switch()" << endl;
 			canMute = true;
 			canRecord = true;
 		}
@@ -262,8 +299,9 @@ Mixer_ALSA::openMixer()
 		    // Everything unknown is handled as switch
 		    cc = MixDevice::SWITCH;
 		}
+	    } // is ordinary mixer element (NOT an enum)
 
-		MixDevice* mdw =
+		MixDevice* md =
 		    new MixDevice( mixerIdx,
 				   *vol,
 					canRecord,
@@ -271,7 +309,22 @@ Mixer_ALSA::openMixer()
 				   snd_mixer_selem_id_get_name( sid ),
 				   ct,
 				   cc );
-		m_mixDevices.append( mdw );
+        if (!masterChosen && ct==MixDevice::VOLUME) {
+           // Determine a nicer MasterVolume
+           m_recommendedMaster = md;
+           masterChosen = true;
+        }
+
+		if ( enumList.count() > 0 ) {
+		  int maxEnumId= enumList.count();
+		  QPtrList<QString>& enumValuesRef = md->enumValues(); // retrieve a ref
+		  for (int i=0; i<maxEnumId; i++ ) {
+		    // we have an enum. Lets set the names of the enum items in the MixDevice
+		    // the enum names are assumed to be static!
+		    enumValuesRef.append(enumList.at(i) );
+		  }
+		}
+		m_mixDevices.append( md );
 		//kdDebug(67100) << "ALSA create MDW, vol= " << *vol << endl;
 		delete vol;
 	    } // virginOpen
@@ -280,13 +333,18 @@ Mixer_ALSA::openMixer()
 		MixDevice* md = m_mixDevices.at( mixerIdx );
 		if( !md )
 		{
-		    return ERR_INCOMPATIBLESET;
+		    return Mixer::ERR_INCOMPATIBLESET;
 		}
 		writeVolumeToHW( mixerIdx, md->getVolume() );
 	    } // !virginOpen
     } // for all elems
 
+    /**************************************************************************************
     // If no devices are supported by this soundcard, return "NO Devices"
+       It is VERY important to return THIS error code, so that the caller knows, that the
+       the device exists.
+       This is used for scanning for existing soundcard devices, see MixerToolBox::initMixer().
+    ***************************************************************************************/
     if ( !validDevice )
     {
 	return Mixer::ERR_NODEV;
@@ -299,15 +357,89 @@ Mixer_ALSA::openMixer()
     // return with success
     m_isOpen = true;
 
+    /* setup for select on stdin and the mixer fd */
+    if ((m_count = snd_mixer_poll_descriptors_count(_handle)) < 0) {
+	kdDebug(67100) << "Mixer_ALSA::poll() , snd_mixer_poll_descriptors_count() err=" <<  m_count << "\n";
+	return Mixer::ERR_OPEN;
+    }
+
+    //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 2\n";
+
+    m_fds = (struct pollfd*)calloc(m_count, sizeof(struct pollfd));
+    if (m_fds == NULL) {
+	kdDebug(67100) << "Mixer_ALSA::poll() , calloc() = null" << "\n";
+        return Mixer::ERR_OPEN;
+    }
+
+    m_fds->events = POLLIN;
+    if ((err = snd_mixer_poll_descriptors(_handle, m_fds, m_count)) < 0) {
+	kdDebug(67100) << "Mixer_ALSA::poll() , snd_mixer_poll_descriptors_count() err=" <<  err << "\n";
+        return Mixer::ERR_OPEN;
+    }
+    if (err != m_count) {
+	kdDebug(67100) << "Mixer_ALSA::poll() , snd_mixer_poll_descriptors_count() err=" << err << " m_count=" <<  m_count << "\n";
+        return Mixer::ERR_OPEN;
+    }
+
     return 0;
 }
 
+void Mixer_ALSA::prepareSignalling( Mixer *mixer )
+{
+    assert( !m_sns );
+
+    m_sns = new QSocketNotifier*[m_count];
+    for ( int i = 0; i < m_count; ++i )
+    {
+        kdDebug() << "socket " << i << endl;
+        m_sns[i] = new QSocketNotifier(m_fds[i].fd, QSocketNotifier::Read);
+        mixer->connect(m_sns[i], SIGNAL(activated(int)), mixer, SLOT(readSetFromHW()));
+    }
+}
+
 
 int
-Mixer_ALSA::releaseMixer()
+Mixer_ALSA::close()
 {
-	int ret = snd_mixer_close( _handle );
-	return ret;
+  int ret=0;
+  m_isOpen = false;
+  if ( _handle != 0 )
+  {
+    //kdDebug(67100) << "IN  Mixer_ALSA::close()" << endl;
+    snd_mixer_free ( _handle );
+    if ( ( ret = snd_mixer_detach ( _handle, devName.latin1() ) ) < 0 )
+    {
+        kdDebug(67100) << "snd_mixer_detach err=" << snd_strerror(ret) << endl;
+    }
+    int ret2 = 0;
+    if ( ( ret2 = snd_mixer_close ( _handle ) ) < 0 )
+    {
+        kdDebug(67100) << "snd_mixer_close err=" << snd_strerror(ret2) << endl;
+	if ( ret == 0 ) ret = ret2; // no error before => use current error code
+    }
+
+    _handle = 0;
+    //kdDebug(67100) << "OUT Mixer_ALSA::close()" << endl;
+
+  }
+
+  mixer_elem_list.clear();
+  mixer_sid_list.clear();
+  m_mixDevices.clear();
+
+  if ( m_fds )
+      free( m_fds );
+  m_fds = 0;
+
+  if ( m_sns )
+  {
+      for ( int i = 0; i < m_count; i++ )
+          delete m_sns[i];
+      delete [] m_sns;
+      m_sns = 0;
+  }
+
+  return ret;
 }
 
 
@@ -315,66 +447,49 @@ snd_mixer_elem_t* Mixer_ALSA::getMixerElem(int devnum) {
 	snd_mixer_elem_t* elem = 0;
 	if ( int( mixer_sid_list.count() ) > devnum ) {
 		snd_mixer_selem_id_t * sid = mixer_sid_list[ devnum ];
+		// The next line (hopefully) only finds selem's, not elem's.
 		elem = snd_mixer_find_selem(_handle, sid);
 
 		if ( elem == 0 ) {
+			// !! Check, whether the warning should be omitted. Probably
+			//    Route controls are non-simple elements.
 			kdDebug(67100) << "Error finding mixer element " << devnum << endl;
 		}
 	}
 	return elem;
-//	return mixer_elem_list[ devnum ];
+
+/*
+ I would have liked to use the following trivial implementation instead of the
+ code above. But it will also return elem's. which are not selem's. As there is
+ no way to check an elem's type (e.g. elem->type == SND_MIXER_ELEM_SIMPLE), callers
+ of getMixerElem() cannot check the type. :-(
+	snd_mixer_elem_t* elem = mixer_elem_list[ devnum ];
+	return elem;
+ */
 }
 
-bool Mixer_ALSA::prepareUpdate() {
+bool Mixer_ALSA::prepareUpdateFromHW()
+{
+    if ( !m_fds )
+	return false;
+
     //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 1\n";
-    if ( _initialUpdate ) {
-        // make sure the very first call to prepareUpdate() returns "true". Otherwise kmix will
-        // show wrong values until a mixer change happens.
-        _initialUpdate = false;
-        return true;
-    }
-    bool updated = false;
-    struct pollfd  *fds;
-    unsigned short revents;
-    int count, err;
-
-/* setup for select on stdin and the mixer fd */
-    if ((count = snd_mixer_poll_descriptors_count(_handle)) < 0) {
-	kdDebug(67100) << "Mixer_ALSA::poll() , snd_mixer_poll_descriptors_count() err=" <<  count << "\n";
-	return false;
-    }
-
-    //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 2\n";
-    
-    fds = (struct pollfd*)calloc(count + 1, sizeof(struct pollfd));
-    if (fds == NULL) {
-	kdDebug(67100) << "Mixer_ALSA::poll() , calloc() = null" << "\n";
-	return false;
-    }
-
-    fds->events = POLLIN;
-    if ((err = snd_mixer_poll_descriptors(_handle, fds, count)) < 0) {
-	kdDebug(67100) << "Mixer_ALSA::poll() , snd_mixer_poll_descriptors_count() err=" <<  err << "\n";
-	return false;
-    }
-    if (err != count) {
-	kdDebug(67100) << "Mixer_ALSA::poll() , snd_mixer_poll_descriptors_count() err=" << err << " count=" <<  count << "\n";
-	    return false;
-    }
 
     // Poll on fds with 10ms timeout
     // Hint: alsamixer has an infinite timeout, but we cannot do this because we would block
     // the X11 event handling (Qt event loop) with this.
     //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 3\n";
-    int finished = poll(fds, count, 10);
+    int finished = poll(m_fds, m_count, 10);
     //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 4\n";
 
+    bool updated = false;
     if (finished > 0) {
-    //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 5\n";
+        //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 5\n";
 
-	if (snd_mixer_poll_descriptors_revents(_handle, fds, count, &revents) >= 0) {
-    //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 6\n";
+        unsigned short revents;
 
+        if (snd_mixer_poll_descriptors_revents(_handle, m_fds, m_count, &revents) >= 0) {
+            //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 6\n";
 
 	    if (revents & POLLNVAL) {
 		kdDebug(67100) << "Mixer_ALSA::poll() , Error: poll() returns POLLNVAL\n";
@@ -385,19 +500,15 @@ bool Mixer_ALSA::prepareUpdate() {
 		return false;
 	    }
 	    if (revents & POLLIN) {
-    //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 7\n";
+                //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 7\n";
 
 		snd_mixer_handle_events(_handle);
                 updated = true;
 	    }
 	}
+
     }
-
-    //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() 8\n";
-
-
-    // !!! memory leak under any error condition
-    free(fds);
+    //kdDebug(67100) << "Mixer_ALSA::prepareUpdate() " << updated << endl;;
     return updated;
 }
 
@@ -469,7 +580,7 @@ Mixer_ALSA::setRecsrcHW( int devnum, bool on )
 		if ( ret != 0 ) {
 			kdDebug(67100) << "snd_mixer_selem_get_capture_switch() failed 1\n";
 		}
-	
+
 		ret = snd_mixer_selem_set_capture_switch_all( elem, sw );
                 if ( ret != 0 ) {
                         kdDebug(67100) << "snd_mixer_selem_set_capture_switch_all() failed 2: errno=" << ret << "\n";
@@ -482,7 +593,7 @@ Mixer_ALSA::setRecsrcHW( int devnum, bool on )
 #ifdef ALSA_SWITCH_DEBUG
 		kdDebug(67100) << "Mixer_ALSA::setRecsrcHW(" << devnum <<  "," << on << ")joined. Before=" << before << " Set=" << sw << " After=" << after <<"\n";
 #endif
-		
+
 	}
 	else
 	{
@@ -500,8 +611,62 @@ Mixer_ALSA::setRecsrcHW( int devnum, bool on )
 	kdDebug(67100) << "EXIT Mixer_ALSA::setRecsrcHW(" << devnum << "," << on <<  ")\n";
 #endif
 	return false; // we should always return false, so that other devnum's get updated
+	// The ALSA polling has been implemented some time ago. So it should be safe to
+	// return "true" here.
+	// The other devnum's Rec-Sources won't get update by KMix code, but ALSA will send
+	// us an event, if neccesary. But OTOH it is possibly better not to trust alsalib fully,
+        // because the old code is working also well (just takes more processing time).
+	// return true;
 }
 
+/**
+ * Sets the ID of the currently selected Enum entry.
+ * Warning: ALSA supports to have different enums selected on each channel
+ *          of the SAME snd_mixer_elem_t. KMix does NOT support that and
+ *          always sets both channels (0 and 1).
+ */
+void Mixer_ALSA::setEnumIdHW(int mixerIdx, unsigned int idx) {
+	//kdDebug(67100) << "Mixer_ALSA::setEnumIdHW(" << mixerIdx << ", idx=" << idx << ") 1\n";
+        snd_mixer_elem_t *elem = getMixerElem( mixerIdx );
+        if ( elem==0 || ( !snd_mixer_selem_is_enumerated(elem)) )
+        {
+                return;
+        }
+
+	//kdDebug(67100) << "Mixer_ALSA::setEnumIdHW(" << mixerIdx << ", idx=" << idx << ") 2\n";
+	int ret = snd_mixer_selem_set_enum_item(elem,SND_MIXER_SCHN_FRONT_LEFT,idx);
+        if (ret < 0) {
+           kdError(67100) << "Mixer_ALSA::setEnumIdHW(" << mixerIdx << "), errno=" << ret << "\n";
+        }
+	snd_mixer_selem_set_enum_item(elem,SND_MIXER_SCHN_FRONT_RIGHT,idx);
+	// we don't care about possible error codes on channel 1
+        return;
+}
+
+/**
+ * Return the ID of the currently selected Enum entry.
+ * Warning: ALSA supports to have different enums selected on each channel
+ *          of the SAME snd_mixer_elem_t. KMix does NOT support that and
+ *          always shows the value of the first channel.
+ */
+unsigned int Mixer_ALSA::enumIdHW(int mixerIdx) {
+	snd_mixer_elem_t *elem = getMixerElem( mixerIdx );
+        if ( elem==0 || ( !snd_mixer_selem_is_enumerated(elem)) )
+        {
+                return 0;
+        }
+
+	unsigned int idx = 0;
+	int ret = snd_mixer_selem_get_enum_item(elem,SND_MIXER_SCHN_FRONT_LEFT,&idx);
+	//kdDebug(67100) << "Mixer_ALSA::enumIdHW(" << mixerIdx << ") idx=" << idx << "\n";
+	if (ret < 0) {
+	   idx = 0;
+	   kdError(67100) << "Mixer_ALSA::enumIdHW(" << mixerIdx << "), errno=" << ret << "\n";
+	}
+	return idx;
+}
+
+
 int
 Mixer_ALSA::readVolumeFromHW( int mixerIdx, Volume &volume )
 {
@@ -513,7 +678,7 @@ Mixer_ALSA::readVolumeFromHW( int mixerIdx, Volume &volume )
 	{
 		return 0;
 	}
-		
+
 
 	// *** READ PLAYBACK VOLUMES *************
 	if ( snd_mixer_selem_has_playback_volume( elem ) )
@@ -543,19 +708,17 @@ Mixer_ALSA::readVolumeFromHW( int mixerIdx, Volume &volume )
 	    else
 	    {
 		int ret = snd_mixer_selem_get_capture_volume( elem, SND_MIXER_SCHN_FRONT_RIGHT, &right );
-		if ( ret != 0 ) kdDebug(67100) << "readVolumeFromHW(" << mixerIdx << ") [has_capture_volume,R] failed, errno=" << ret << endl;			
+		if ( ret != 0 ) kdDebug(67100) << "readVolumeFromHW(" << mixerIdx << ") [has_capture_volume,R] failed, errno=" << ret << endl;
 		volume.setVolume( Volume::LEFT , left );
 		volume.setVolume( Volume::RIGHT, right );
 	    }
 	}
 
+        //kdDebug() << "snd_mixer_selem_has_playback_volume " << mixerIdx << " " << snd_mixer_selem_has_playback_switch( elem ) << endl;
 	if ( snd_mixer_selem_has_playback_switch( elem ) )
 	{
 	    snd_mixer_selem_get_playback_switch( elem, SND_MIXER_SCHN_FRONT_LEFT, &elem_sw );
-	    if( elem_sw == 0 )
-		volume.setMuted(true);
-	    else
-		volume.setMuted(false);
+            volume.setMuted( elem_sw == 0 );
 	}
 
 	return 0;
@@ -565,18 +728,18 @@ int
 Mixer_ALSA::writeVolumeToHW( int devnum, Volume& volume )
 {
 	int left, right;
-	
+
 	snd_mixer_elem_t *elem = getMixerElem( devnum );
 	if ( !elem )
 	{
 		return 0;
 	}
-	
-	// --- VOLUME  - WE HAVE JUST ONE TYPE OF VOLUME A TIME, 
+
+	// --- VOLUME  - WE HAVE JUST ONE TYPE OF VOLUME A TIME,
 	// CAPTURE OR PLAYBACK, SO IT"S JUST USE VOLUME ------------
 	left = volume[ Volume::LEFT ];
 	right = volume[ Volume::RIGHT ];
-	
+
 	if (snd_mixer_selem_has_playback_volume( elem ) ) {
 		snd_mixer_selem_set_playback_volume ( elem, SND_MIXER_SCHN_FRONT_LEFT, left );
 		if ( ! snd_mixer_selem_is_playback_mono ( elem ) )
@@ -587,7 +750,7 @@ Mixer_ALSA::writeVolumeToHW( int devnum, Volume& volume )
 		if ( ! snd_mixer_selem_is_playback_mono ( elem ) )
 			snd_mixer_selem_set_capture_volume ( elem, SND_MIXER_SCHN_FRONT_RIGHT, right );
 	}
-	
+
 	if ( snd_mixer_selem_has_playback_switch( elem ) )
 	{
 		int sw = 0;
@@ -605,17 +768,17 @@ Mixer_ALSA::errorText( int mixer_error )
 	QString l_s_errmsg;
 	switch ( mixer_error )
 	{
-		case ERR_PERM:
+		case Mixer::ERR_PERM:
 			l_s_errmsg = i18n("You do not have permission to access the alsa mixer device.\n" \
 					"Please verify if all alsa devices are properly created.");
       break;
-		case ERR_OPEN:
+		case Mixer::ERR_OPEN:
 			l_s_errmsg = i18n("Alsa mixer cannot be found.\n" \
 					"Please check that the soundcard is installed and the\n" \
 					"soundcard driver is loaded.\n" );
 			break;
 		default:
-			l_s_errmsg = Mixer::errorText( mixer_error );
+			l_s_errmsg = Mixer_Backend::errorText( mixer_error );
 	}
 	return l_s_errmsg;
 }
