@@ -1,5 +1,5 @@
 /*
-  generic interactive controler 
+  generic interactive controller 
   Copyright (C) 1998  Martin Vogt
 
   This program is free software; you can redistribute it and/or modify
@@ -13,9 +13,9 @@
 
 
 #include "xplayer_control.h"
+#include <iostream>
 
-
-
+using namespace std;
 
 
 extern "C" void control_xplayer() {
